>From 366d68527f58085d5761b43d76a4cddf646342a9 Mon Sep 17 00:00:00 2001
From: Tadas Slotkus <devtadas@gmail.com>
Date: Sat, 13 Aug 2011 00:45:33 +0300
Subject: [carFl mem 01/21] Fix check_erased_range. (smallchunks)

Use two passed already allocated buffers, check in small chunks.
Introduce macro define for selecting chunk size. Also change version,
so that logs won't confuse mainline developers and this eliminates
getting version from makefile which won't be in coreboot's make system.

Signed-off-by: Tadas Slotkus <devtadas@gmail.com>
---
 flash.h    |    7 +++++++
 flashrom.c |   23 +++++++++++------------
 2 files changed, 18 insertions(+), 12 deletions(-)

diff --git a/flash.h b/flash.h
index 4b1cca2..7f51004 100644
--- a/flash.h
+++ b/flash.h
@@ -73,6 +73,13 @@ enum chipbustype {
 #define NUM_ERASEREGIONS 5
 
 /*
+ * The size of one allocated buffer used for temporary data storage.
+ * These are two actually. You may want to experiment in other power of two.
+ * Maybe use your flash page size here?
+ */
+#define BUF_SIZE 256
+
+/*
  * How many different erase functions do we have per chip?
  * Atmel AT25FS010 has 6 different functions.
  */
diff --git a/flashrom.c b/flashrom.c
index 07cfdd9..2ac8d26 100644
--- a/flashrom.c
+++ b/flashrom.c
@@ -38,7 +38,7 @@
 #include "flashchips.h"
 #include "programmer.h"
 
-const char flashrom_version[] = FLASHROM_VERSION;
+const char flashrom_version[] = "carFl";
 char *chip_to_probe = NULL;
 int verbose = 0;
 
@@ -727,19 +727,17 @@ static unsigned int count_usable_erasers(const struct flashchip *flash)
 }
 
 /* start is an offset to the base address of the flash chip */
-int check_erased_range(struct flashchip *flash, int start, int len)
+int check_erased_range(struct flashchip *flash, int start, int len, uint8_t *cmpbuf, uint8_t *curcontents)
 {
-	int ret;
-	uint8_t *cmpbuf = malloc(len);
+	int starthere = start, lenhere = BUF_SIZE;
 
-	if (!cmpbuf) {
-		msg_gerr("Could not allocate memory!\n");
-		exit(1);
+	for (; starthere < start + len; starthere += lenhere) {
+		memset(cmpbuf, 0xff, lenhere);
+		flash->read(flash, curcontents, starthere, lenhere);
+		if (memcmp(curcontents, cmpbuf, lenhere))
+			return -1;
 	}
-	memset(cmpbuf, 0xff, len);
-	ret = verify_range(flash, cmpbuf, start, len, "ERASE");
-	free(cmpbuf);
-	return ret;
+	return 0;
 }
 
 /*
@@ -1404,6 +1402,7 @@ static int erase_and_write_block_helper(struct flashchip *flash,
 	 */
 	curcontents += start;
 	newcontents += start;
+	uint8_t cmpbuf[BUF_SIZE]; //very temporary, see upcoming patches
 	msg_cdbg(":");
 	/* FIXME: Assume 256 byte granularity for now to play it safe. */
 	if (need_erase(curcontents, newcontents, len, gran)) {
@@ -1411,7 +1410,7 @@ static int erase_and_write_block_helper(struct flashchip *flash,
 		ret = erasefn(flash, start, len);
 		if (ret)
 			return ret;
-		if (check_erased_range(flash, start, len)) {
+		if (check_erased_range(flash, start, len, cmpbuf, curcontents)) {
 			msg_cerr("ERASE FAILED!\n");
 			return -1;
 		}
-- 
1.7.6

